<?php

namespace Drupal\commerce_shipping_colissimo\Api;

use Drupal\commerce_shipping_colissimo\Settings;
use Drupal\Core\Logger\LoggerChannelInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\RequestOptions;
use Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Normalizer\PropertyNormalizer;
use Symfony\Component\Serializer\Serializer;

class UrlEncodedClient {
  private ClientInterface $httpClient;

  private Settings $settings;

  private Serializer $serializer;

  private LoggerChannelInterface $logger;

  public function __construct(ClientInterface $httpClient, LoggerChannelInterface $logger, Settings $settings) {
    $this->httpClient = $httpClient;
    $this->settings = $settings;
    $this->serializer = new Serializer([
      /**
       * Handle objects with public properties, note PhpDocExtractor that allows
       * nested object deserialization via PhpDoc
       */
      new PropertyNormalizer(NULL, NULL, new PhpDocExtractor()),
    ], [new JsonEncoder()]);
    $this->logger = $logger;
  }

  /**
   * @template T
   * @param class-string<T>|NULL $returnClass
   * @return T|NULL
   */
  public function query(string $path, ?object $payload = NULL, ?string $returnClass = NULL) {
    $setting = $this->settings->get();
    $url = $setting->getBaseUrl() . $path;

    $options = [
      RequestOptions::HEADERS => [
        'Content-Type' => 'application/x-www-form-urlencoded',
        'Accept' => 'application/json',
      ],
    ];

    $setting = $this->settings->get();
    $payload = $this->normalize($payload);
    $payload['login'] = $setting->getUser();
    $payload['password'] = $setting->getPassword();

    $options[RequestOptions::BODY] = http_build_query($payload);

    $httpMethod = 'POST';
    if ($setting->isDebugMode()) {
      $this->logger->debug('Sending %httpMethod request to %url: %options', [
        '%url' => $url,
        '%httpMethod' => $httpMethod,
        '%options' => print_r($options, TRUE),
      ]);
    }

    $httpResponse = $this->httpClient->request($httpMethod, $url, $options);
    $body = $httpResponse->getBody()->getContents();

    if ($setting->isDebugMode()) {
      $this->logger->debug('Received response: %response', [
        '%response' => $body,
      ]);
    }

    if ($returnClass) {
      /** @var T $response */
      $response = $this->serializer->deserialize($body, $returnClass, 'json');
      return $response;
    }
    return NULL;
  }

  private function normalize(?object $object): array {
    if (!$object) {
      return [];
    }
    return $this->serializer->normalize($object);
  }

}
