<?php

namespace Drupal\commerce_shipping_colissimo;

use Drupal\address\Plugin\Field\FieldType\AddressItem;
use Drupal\profile\Entity\ProfileInterface;

class CustomerProfileWrapper {
  private ProfileInterface $profile;

  public function __construct(ProfileInterface $profile) {
    $this->profile = $profile;
  }

  public function save(): void {
    $this->profile->save();
  }

  public function getAddress(): ?array {
    return $this->getAddressField()->getValue();
  }

  public function getAddressField(): ?AddressItem {
    $field = $this->profile->get('address')->first();
    if (!$field) {
      return NULL;
    }
    assert($field instanceof AddressItem);
    return $field;
  }

  public function setAddress(?array $values): void {
    $this->getAddressField()->setValue($values);
  }

  public function getRelayId(): ?int {
    return $this->profile->getData('relay_id');
  }

  public function setRelayId(?int $id): void {
    $this->profile->setData('relay_id', $id);
  }

  public function clearAddressBook(): void {
    $this->profile->unsetData('address_book_profile_id');
    $this->profile->unsetData('copy_to_address_book');
  }

  public function getMobilePhone(Setting $setting): ?string {
    if (!$setting->getCustomerProfilePhoneField()) {
      return $this->profile->getData('colissimo_mobile_phone');
    }
    return $this->profile->get($setting->getCustomerProfilePhoneField())->getString();
  }

  public function setMobilePhone(?string $phone, Setting $setting): void {
    $mobilePhone = new MobilePhoneNumber($phone);
    if (!$setting->getCustomerProfilePhoneField()) {
      $this->profile->setData('colissimo_mobile_phone', $mobilePhone->format());
      return;
    }
    $this->profile->get($setting->getCustomerProfilePhoneField())->setValue($mobilePhone->format());
  }

}
